#!/bin/bash
#
# Compiles all \$EPAGES_CARTRIDGES/CARTRIDGE/Templates/**.html to
# language-dependent ctmpl files OUTDIR/LANG/CARTRIDGE/**.LANG.ctmpl

# get SCRIPT_NAME
SCRIPT_NAME=${0##*/}

# set EPAGES_PROJECT to epages6 unless defined
EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}
# source /etc/default/$EPAGES_PROJECT unless $EPAGES is set
[[ -n $EPAGES ]] || . /etc/default/$EPAGES_PROJECT

########################################################################
# USAGE
########################################################################

usage() {
  [[ -n $@ ]] && echo "Error: $@"
  echo "Usage: $SCRIPT_NAME [-c ...] -l ... [-o ...] [-p ...] [-v] [-n]"
  echo "Description:"
  echo "  Compiles all \$EPAGES_CARTRIDGES/CARTRIDGE/Templates/**.html to"
  echo "  language-dependent ctmpl files OUTDIR/LANG/CARTRIDGE/**.LANG.ctmpl"
  echo "Options:"
  echo "  -c  : create ctmpls for cartridge(s) (space-sep. list or default: ALL)"
  echo "  -l  : create ctmpls for language(s) (space-sep. list)"
  echo "  -o  : target directory for ctmpl files (default: .)"
  echo "  -n  : dry-run, perform a trial run with no changes made"
  echo "  -v  : verbose output"
  echo "  -p  : maximum number of parallel processes (default: 1)"
  echo "Examples:"
  echo "  $SCRIPT_NAME -n -v -c ALL -l 'de en' -o /tmp/Templates -p 5"
  echo "  $SCRIPT_NAME -c DE_EPAGES::Core -l 'de en' -o /tmp/Templates"
  echo "  $SCRIPT_NAME -c DE_EPAGES/Core -l 'de en' -o /tmp/Templates"
  exit 1
}

########################################################################
# ARGUMENTS
########################################################################

OUTDIR=.
MAX_PARALLEL=1
CARTRIDGE_LIST=ALL
QUIET=-nooutput

unset LANG_LIST VERBOSE DRY_RUN

# get options
while getopts c:l:o:p:vn c ; do
  case $c in
    c) CARTRIDGE_LIST=$OPTARG ;;
    l) LANG_LIST=$OPTARG ;;
    o) OUTDIR=$OPTARG ;;
    p) MAX_PARALLEL=$OPTARG ;;
    v) QUIET= ;;
    n) DRY_RUN=--dry-run ;;
    *) usage ;;
  esac
done

# options must not be empty
[[ -z $CARTRIDGE_LIST ]] && usage "Option -c 'CARTRIDGE_LIST' empty."
[[ -z $LANG_LIST ]] && usage "Option -l 'LANG_LIST' empty."
[[ -z $OUTDIR ]] && usage "Option -o 'OUTDIR' empty."
[[ -z $MAX_PARALLEL ]] && usage "Option -p 'MAX_PARALLEL' empty."

# no parallel run for only one cartridge
[[ $CARTRIDGE_LIST = ${CARTRIDGE_LIST// /} ]] && MAX_PARALLEL=1

# MAX_PARALLEL must consist only of numbers
[[ -n ${MAX_PARALLEL//[0-9]/} ]] && usage "Option -p MAX_PARALLEL must be number." 

# parallel program must exist
if [[ $MAX_PARALLEL -gt 1 ]] ; then
  parallel --version > /dev/null || usage "GNU parallel missing; must be installed."
fi

# create output directory
mkdir -p -m 755 "$OUTDIR" || usage
OUTDIR=$(readlink -f "$OUTDIR")

# CARTRIDGES_DIR must exist
pushd "$EPAGES_CARTRIDGES" >/dev/null || usage "EPAGES_CARTRIDGES=$EPAGES_CARTRIDGES not found."

# cartridges in EPAGES_CARTRIDGES must exist
if [[ $CARTRIDGE_LIST = ALL ]] ; then
  CARTRIDGE_LIST=$(find */*/Templates -maxdepth 0 -printf "%h ") || usage
else
  CARTRIDGE_LIST=${CARTRIDGE_LIST//:://}
  ls -d $CARTRIDGE_LIST >/dev/null || usage
fi

########################################################################
# EXECUTION
########################################################################

# perl script createCtmpls4Cartridge.pl must exist
PERL_SCRIPT=$EPAGES_CARTRIDGES/DE_EPAGES/Dictionary/Scripts/createCtmpls4Cartridge.pl
ls "$PERL_SCRIPT" >/dev/null || usage "Perl script missing: $PERL_SCRIPT"

# make error.log empty
if [[ -f $EPAGES_LOG/error.log ]] ; then
  cp -p $EPAGES_LOG/error.log $EPAGES_LOG/error.log-$(date '+%Y.%m.%d-%H.%M.%S')
  cat /dev/null > $EPAGES_LOG/error.log
fi

#-----------------------------------------------------------------------
# run createCtmpls4Cartridge.pl
#-----------------------------------------------------------------------

if [[ $MAX_PARALLEL = 1 ]] ; then
  for cartridge in $CARTRIDGE_LIST ; do
    echo "$PERL_SCRIPT $QUIET -cartridge $cartridge -outdir '$OUTDIR' -lang '$LANG_LIST'"
    [[ -n $DRY_RUN ]] && continue
    $PERL "$PERL_SCRIPT" $QUIET -cartridge $cartridge -outdir "$OUTDIR" -lang "$LANG_LIST"
  done
else
  CMD="$PERL '$PERL_SCRIPT' -cartridge '{}' -outdir '$OUTDIR' -lang '$LANG_LIST'"
  parallel $DRY_RUN --no-notice -t -j $MAX_PARALLEL "$CMD -nooutput"  ::: $CARTRIDGE_LIST
fi

#-----------------------------------------------------------------------
# show not replaced tags
#-----------------------------------------------------------------------

sed "1,/### $SCRIPT_NAME ###/d" $EPAGES_LOG/error.log \
| grep '^tag .* was not replaced' \
| while read t tag in l lang was not replaced in t template other
do
  [[ $template =~ /Diagnostics/ ]] && continue
  [[ $template =~ /Presentation/Templates/SimpleError.html ]] && continue
  template=${template%.}
  echo "WARNING: tag not replaced: ${tag%%[*} $lang ${template#$EPAGES_CARTRIDGES/}"
done

#-----------------------------------------------------------------------
# show missing languages
#-----------------------------------------------------------------------

sed "1,/### $SCRIPT_NAME ###/d" $EPAGES_LOG/error.log \
| grep '^language unknown .* on template' \
| while read l unknown lang on t template other
do
  [[ $template =~ /Diagnostics/ ]] && continue
  [[ $template =~ /Presentation/Templates/SimpleError.html ]] && continue
  DICT=DE_EPAGES/Dictionary/Templates/Dictionary.$lang.xml
  echo "WARNING: language '$lang' unknown, file missing: $DICT"
done | sort | uniq

########################################################################
# END
########################################################################
exit 0
