#========================================================================================
# package      DE_EPAGES::SageOne::t::unit::OrderService_Delete
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::SageOne::API::Webservice::OrderService
#========================================================================================
package DE_EPAGES::SageOne::t::unit::OrderService_Delete;
use base DE_EPAGES::Test::API::UnitTest::Test;
use DE_EPAGES::Core::API::Error qw (Error);
use Test::More;
use Test::Exception;
use HTTP::Status qw (HTTP_OK);
use DE_EPAGES::SageOne::API::Constants qw (
    SAGE_ORDER_DELETE_PATH_TEMPLATE
);
use strict;

sub ErrorDefinition {
    return {
        'JsonParserError' => 'malformed JSON string',
    };
}

sub test {
    my $self = shift;

    $self->_buildMocks();

    $self->test_deleteSalesInvoiceResponseOk();
    $self->test_deleteSalesInvoiceWebserviceError();
    $self->test_deleteSalesInvoiceUnknownError();
    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_deleteSalesInvoiceResponseOk {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $OrderGuid = '1002';
    my $SUT = $self->memory()->get('SUTSResponseOk');

    lives_ok( sub { $SUT->deleteSalesInvoice($OrderGuid); }, "$SubTestName - test if deleteSalesInvoice lives");
}

sub test_deleteSalesInvoiceWebserviceError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $OrderGuid = '1002';
    my $SUT = $self->memory()->get('SUTWebserviceError');

    throws_ok( sub {  $SUT->deleteSalesInvoice($OrderGuid); },
                   qr/WebserviceError/,
                   "$SubTestName - test if deleteSalesInvoice throws WebserviceError"
         );
}

sub test_deleteSalesInvoiceUnknownError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $OrderGuid = '1002';
    my $SUT = $self->memory()->get('SUTUnknownError');

    throws_ok( sub {  $SUT->deleteSalesInvoice($OrderGuid); },
                   qr/Unknown/,
                   "$SubTestName - test if deleteSalesInvoice throws Unknown"
         );
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildMocks {
    my $self = shift;

    my $System = $self->mockFactory->buildSite({
        'EPagesJBaseURL' => 'http://mydomain.epages6.com:8088/rs/'
    })->getMockObject();

    my $Shop = $self->mockFactory()->buildShop({
            'GUID' => 'MyShopAlias',
            'RESTAPIURL' => 'http://external/base/url/',
            'Site' => $System
        })->getMockObject();


    my $SUTSResponseOk = $self->_createSUTSuccessful($Shop, $System, $self->_createResponse(HTTP_OK, ''));
    my $SUTWebserviceError = $self->_createSUTWebserviceError($Shop, $System);
    my $SUTUnknownError = $self->_createSUTUnknownError($Shop, $System);

    $self->memory()->add('SUTSResponseOk', $SUTSResponseOk);
    $self->memory()->add('SUTWebserviceError', $SUTWebserviceError);
    $self->memory()->add('SUTUnknownError', $SUTUnknownError);
}
#----------------------------------------------------------------------------------------
sub _createSUTSuccessful {
    my $self = shift;
    my ($Shop, $System, $Response) = @_;

    my $aParameterList = [$Shop];
    my $hHeaders = {};
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                 'DE_EPAGES::SageOne::API::Webservice::OrderService', $aParameterList);

    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    $Mockify->addMockWithReturnValueAndParameterCheck('sendDeleteRequest', $Response,
       [{'string' => 'http://epagesj-endpoint.com'}, {'hashref'=> $hHeaders}]
    );

    $Mockify->addMockWithReturnValueAndParameterCheck('decodeResponse', $Response->content(),
       [{'object'=> 'HTTP::Response'}]
    );

    return $Mockify->getMockObject();
}

sub _createSUTWebserviceError {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::OrderService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    $Mockify->addMock('sendDeleteRequest', sub{ Error('WebserviceError');});

    return $Mockify->getMockObject();
}

sub _createSUTUnknownError {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::OrderService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    $Mockify->addMock('sendDeleteRequest', sub{ Error('Unknown');});

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShopUriBuilder {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop, $System];

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::Shop::API::ShopUriBuilder',
                   $aParameterList
              );

    my $hQueryParams = {};
    my $PathTemplate = SAGE_ORDER_DELETE_PATH_TEMPLATE;
    my $hPathParams = {'shopGuid'=>'MyShopAlias', 'orderGuid' => '1002'};
    $Mockify->addMockWithReturnValueAndParameterCheck('buildPrivateUri', 'http://epagesj-endpoint.com', [
        {'string' => $PathTemplate}, {'hashref' => $hPathParams}, {'hashref' => $hQueryParams}
    ]);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createResponse {
    my $self = shift;
    my ($Code, $Content) = @_;
    my $Response = HTTP::Response->new($Code, 'all is fine');
    $Response->header('content-type' => 'application/json');
    $Response->content($Content);
    return $Response;
}
__PACKAGE__->runScript();