#========================================================================================
# package      DE_EPAGES::SageOne::t::unit::OrderService_Status
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::SageOne::API::Webservice::OrderService
#========================================================================================
package DE_EPAGES::SageOne::t::unit::OrderService_Status;
use base DE_EPAGES::Test::API::UnitTest::Test;
use DE_EPAGES::Core::API::Error qw (Error);
use Test::More;
use Test::Exception;
use HTTP::Status qw (HTTP_OK HTTP_NOT_MODIFIED);
use DE_EPAGES::SageOne::API::Constants qw (
    SAGE_ORDER_STATUS_PATH_TEMPLATE
);
use strict;

sub ErrorDefinition {
    return {
        'JsonParserError' => 'malformed JSON string',
    };
}

sub test {
    my $self = shift;

    $self->_buildMocks();

    $self->test_getOrderStatusResponseOk();
    $self->test_getOrderStatusResponseNoContent();
    $self->test_getOrderStatusResponseError();
    $self->test_getOrderStatusServiceError();

    $self->test_getOrderStatusJsonParserError();
    $self->test_getOrderStatusWebserviceError();
    $self->test_getOrderStatusUnknownError();
    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_getOrderStatusResponseOk {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $OrderGuid = '1002';
    my $SUT = $self->memory()->get('SUTSResponseOk');
    my $DTO = $SUT->getOrderStatus($OrderGuid);

    is($DTO->shopId(), 'ConnectedShop', "$SubTestName - test if request was successful and return correct shopId");
    is($DTO->orderId(), '1002', "$SubTestName - test if request was successful and return correct OrderGuid");
    is($DTO->sageSalesInvoiceReference(), '700502', "$SubTestName - test if request was successful and return correct sageSalesInvoiceReference");
    is($DTO->sagePaymentReference(), undef, "$SubTestName - test if request was successful and return correct sagePaymentReference");
    is($DTO->status(), 'published', "$SubTestName - test if request was successful and return correct status");
    is($DTO->statusMessage(), undef, "$SubTestName - test if request was successful and return correct statusMessage");
    is($DTO->statusChanged(), '05.16.2016 14:31:29', "$SubTestName - test if request was successful and return correct statusChanged");
}

sub test_getOrderStatusResponseNoContent {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $OrderGuid = '1002';
    my $SUT = $self->memory()->get('SUTResponseNotPublished');
    my $DTO = $SUT->getOrderStatus($OrderGuid);

    is($DTO->shopId(), 'ConnectedShop', "$SubTestName - test if request was successful and return correct shopId");
    is($DTO->orderId(), '1002', "$SubTestName - test if request was successful and return correct OrderGuid");
    is($DTO->status(), 'not_published', "$SubTestName - test if request was successful and return correct status");
}

sub test_getOrderStatusResponseError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $OrderGuid = '1002';
    my $SUT = $self->memory()->get('SUTSResponseError');
    throws_ok(sub { $SUT->getOrderStatus($OrderGuid); },
              qr/WebserviceError: Missing Field shop_id/,
              "$SubTestName - test if getOrderStatus throws ResponseError with missing field shop_id");
}

sub test_getOrderStatusServiceError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $OrderGuid = '1002';
    my $SUT = $self->memory()->get('SUTSResponseUnknown');

    throws_ok( sub {  $SUT->getOrderStatus($OrderGuid); },
                   qr/WebserviceError/,
                   "$SubTestName - test if getOrderStatus throws Unknown error"
         );
}

sub test_getOrderStatusJsonParserError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $OrderGuid = '1002';
    my $SUT = $self->memory()->get('SUTJsonParserError');

    throws_ok( sub {  $SUT->getOrderStatus($OrderGuid); },
                   qr/WebserviceError: malformed JSON/,
                   "$SubTestName - test if getOrderStatus throws JsonParserError"
         );
}

sub test_getOrderStatusWebserviceError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $OrderGuid = '1002';
    my $SUT = $self->memory()->get('SUTWebserviceError');

    throws_ok( sub {  $SUT->getOrderStatus($OrderGuid); },
                   qr/WebserviceError/,
                   "$SubTestName - test if getOrderStatus throws WebserviceError"
         );
}

sub test_getOrderStatusUnknownError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $OrderGuid = '1002';
    my $SUT = $self->memory()->get('SUTUnknownError');

    throws_ok( sub {  $SUT->getOrderStatus($OrderGuid); },
                   qr/Unknown/,
                   "$SubTestName - test if getOrderStatus throws Unknown"
         );
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildMocks {
    my $self = shift;

    my $Shop = $self->mockFactory()->buildShop({
            'GUID' => 'MyShopAlias',
            'RESTAPIURL' => 'http://external/base/url/'
        })->getMockObject();
    my $System = $self->mockFactory->buildSite({
        'EPagesJBaseURL' => 'http://mydomain.epages6.com:8088/rs/'
    })->getMockObject();

    my $SUTSResponseOk = $self->_createSUTSuccessful($Shop, $System, $self->_createResponse(HTTP_OK, $self->_createJsonForDTO()));
    my $SUTResponseNotPublished = $self->_createSUTSuccessful($Shop, $System, $self->_createResponse(HTTP_OK, $self->_createJsonForDTO_NotPublished()));
    my $SUTSResponseError = $self->_createSUTSuccessful($Shop, $System, $self->_createResponse(HTTP_OK, $self->_createJsonForNotDTO()));
    my $SUTSResponseUnknown = $self->_createSUTSuccessful($Shop, $System, $self->_createResponse(HTTP_NOT_MODIFIED));

    my $SUTWebserviceError = $self->_createSUTWebserviceError($Shop, $System);
    my $SUTJsonParserError = $self->_createSUTJsonParserError($Shop, $System);
    my $SUTUnknownError = $self->_createSUTUnknownError($Shop, $System);

    $self->memory()->add('SUTSResponseOk', $SUTSResponseOk);
    $self->memory()->add('SUTResponseNotPublished', $SUTResponseNotPublished);
    $self->memory()->add('SUTSResponseError', $SUTSResponseError);
    $self->memory()->add('SUTSResponseUnknown', $SUTSResponseUnknown);

    $self->memory()->add('SUTWebserviceError', $SUTWebserviceError);
    $self->memory()->add('SUTJsonParserError', $SUTJsonParserError);
    $self->memory()->add('SUTUnknownError', $SUTUnknownError);
}
#----------------------------------------------------------------------------------------
sub _createSUTSuccessful {
    my $self = shift;
    my ($Shop, $System, $Response) = @_;

    my $aParameterList = [$Shop];
    my $hHeaders = {};
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                 'DE_EPAGES::SageOne::API::Webservice::OrderService', $aParameterList);

    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    $Mockify->addMockWithReturnValueAndParameterCheck('sendGetRequest', $Response,
       [{'string' => 'http://epagesj-endpoint.com'}, {'hashref'=> $hHeaders}]
    );

    $Mockify->addMockWithReturnValueAndParameterCheck('decodeResponse', $Response->content(),
       [{'object'=> 'HTTP::Response'}]
    );

    return $Mockify->getMockObject();
}

sub _createSUTWebserviceError {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::OrderService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    $Mockify->addMock('sendGetRequest', sub{ Error('WebserviceError');});

    return $Mockify->getMockObject();
}

sub _createSUTJsonParserError {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::OrderService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    $Mockify->addMockWithReturnValueAndParameterCheck('sendGetRequest', $self->_createResponse(HTTP_OK, 'abc'), [{'string'=>'http://epagesj-endpoint.com'}, {'hashref' => {}}]);

    $Mockify->addMock('decodeResponse', sub{ Error('JsonParserError');});

    return $Mockify->getMockObject();
}

sub _createSUTUnknownError {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::OrderService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    $Mockify->addMock('sendGetRequest', sub{ Error('Unknown');});

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShopUriBuilder {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop, $System];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::Shop::API::ShopUriBuilder',
                   $aParameterList
              );

    my $hQueryParams = {};
    my $PathTemplate = SAGE_ORDER_STATUS_PATH_TEMPLATE;
    my $hPathParams = {'shopGuid'=>'MyShopAlias', 'orderGuid' => '1002'};
    $Mockify->addMockWithReturnValueAndParameterCheck('buildPrivateUri', 'http://epagesj-endpoint.com', [
        {'string' => $PathTemplate}, {'hashref' => $hPathParams}, {'hashref' => $hQueryParams}
    ]);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createResponse {
    my $self = shift;
    my ($Code, $Content) = @_;
    my $Response = HTTP::Response->new($Code, 'all is fine');
    $Response->header('content-type' => 'application/json');
    $Response->content($Content);
    return $Response;
}
#----------------------------------------------------------------------------------------
sub _createJsonForDTO {
    my $self = shift;
    return {
      'shop_id' => 'ConnectedShop',
      'order_id' => '1002',
      'sage_sales_invoice_reference' => '700502',
      'sage_payment_reference' => undef,
      'status' => 'published',
      'status_message' => undef,
      'status_changed' => '05.16.2016 14:31:29'
    };
}

sub _createJsonForDTO_NotPublished {
    my $self = shift;
    return {
      'shop_id' => 'ConnectedShop',
      'order_id' => '1002',
      'status' => 'not_published',
      'sage_sales_invoice_reference' => undef,
      'sage_payment_reference' => undef,
      'status_message' => undef,
      'status_changed' => undef
    };
}

sub _createJsonForNotDTO {
    my $self = shift;
    return {
      'another_shop_id' => 'ConnectedShop'
    };
}
__PACKAGE__->runScript();