#========================================================================================
# package      DE_EPAGES::SageOne::t::unit::ShopConfigService_Get
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::SageOne::API::Webservice::ShopConfigService
#========================================================================================
package DE_EPAGES::SageOne::t::unit::ShopConfigService_Get;
use base DE_EPAGES::Test::API::UnitTest::Test;
use DE_EPAGES::Core::API::Error qw (Error);
use Test::More;
use Test::Exception;
use HTTP::Status qw (HTTP_OK HTTP_NO_CONTENT HTTP_NOT_MODIFIED);
use DE_EPAGES::SageOne::API::Constants qw (SAGE_SHOP_CONFIG_PATH_TEMPLATE);
use HTTP::Response;
use strict;

sub ErrorDefinition {
    return {
        'JsonParserError' => 'malformed JSON string',
    };
}

sub test {
    my $self = shift;

    $self->_buildMocks();

    $self->test_getShopConfig_ResponseOk();
    $self->test_getShopConfig_ResponseNoContent();
    $self->test_getShopConfig_ResponseNotHandled();
    $self->test_getShopConfig_ResponseError();

    $self->test_getShopConfig_JsonParserError();
    $self->test_getShopConfig_WebserviceError();
    $self->test_getShopConfig_UnknownError();
    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_getShopConfig_ResponseOk {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUTResponseOk');
    my $DTO = $SUT->getShopConfig();

    is($DTO->shopId(), 'ConnectedShop', "$SubTestName - test if request was successful and return correct shopId");
    is($DTO->ledgerAccountInvoice(), '2001', "$SubTestName - test if request was successful and return correct ledgerAccountInvoice");
}

sub test_getShopConfig_ResponseNoContent {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUTResponseNoContent');
    throws_ok(sub { $SUT->getShopConfig(); }, qr/ShopConfigNotFound/, "$SubTestName - test if getShopConfig throws ShopConfigNotFound");
}

sub test_getShopConfig_ResponseNotHandled {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUTResponseUnknown');

    throws_ok( sub {  $SUT->getShopConfig(); },
                   qr/WebserviceError: Message=Response status code not handled in getShopConfig/, ## no critic (ProhibitComplexRegexes)
                   "$SubTestName - test if getShopConfig throws WebserviceError when the Status Code wasn't handled."
         );
}

sub test_getShopConfig_ResponseError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUTResponseError');
    throws_ok(sub { $SUT->getShopConfig(); },
              qr/WebserviceError: Missing Field shop_id/,
              "$SubTestName - test if getShopConfig throws ResponseError with missing field shop_id");
}

sub test_getShopConfig_JsonParserError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUTJsonParserError');

    throws_ok( sub {  $SUT->getShopConfig(); },
                   qr/WebserviceError: malformed JSON/,
                   "$SubTestName - test if getShopConfig throws WebserviceError"
         );
}

sub test_getShopConfig_WebserviceError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUTWebserviceError');

    throws_ok( sub {  $SUT->getShopConfig(); },
                   qr/WebserviceError/,
                   "$SubTestName - test if getOrderStatus throws WebserviceError"
         );
}

sub test_getShopConfig_UnknownError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUTUnknownError');

    throws_ok( sub {  $SUT->getShopConfig(); },
                   qr/Unknown/,
                   "$SubTestName - test if getOrderStatus throws Unknown"
         );
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildMocks {
    my $self = shift;

    my $Shop = $self->mockFactory()->buildShop({
            'GUID' => 'MyShopAlias',
            'RESTAPIURL' => 'http://external/base/url/'
        })->getMockObject();
    my $System = $self->mockFactory->buildSite({
        'EPagesJBaseURL' => 'http://mydomain.epages6.com:8088/rs/'
    })->getMockObject();

    my $SUTResponseOk = $self->_createSUTSuccessful($Shop, $System, $self->_createResponse(HTTP_OK, $self->_createJsonForDTO()));
    my $SUTResponseNoContent = $self->_createSUTSuccessful($Shop, $System, $self->_createResponse(HTTP_NO_CONTENT));
    my $SUTResponseError = $self->_createSUTSuccessful($Shop, $System, $self->_createResponse(HTTP_OK, $self->_createJsonForNotDTO()));
    my $SUTResponseUnknown = $self->_createSUTSuccessful($Shop, $System, $self->_createResponse(HTTP_NOT_MODIFIED));

    my $SUTWebserviceError = $self->_createSUTWebserviceError($Shop, $System);
    my $SUTJsonParserError = $self->_createSUTJsonParserError($Shop, $System);
    my $SUTUnknownError = $self->_createSUTUnknownError($Shop, $System);

    $self->memory()->add('SUTResponseOk', $SUTResponseOk);
    $self->memory()->add('SUTResponseNoContent', $SUTResponseNoContent);
    $self->memory()->add('SUTResponseError', $SUTResponseError);
    $self->memory()->add('SUTResponseUnknown', $SUTResponseUnknown);

    $self->memory()->add('SUTWebserviceError', $SUTWebserviceError);
    $self->memory()->add('SUTJsonParserError', $SUTJsonParserError);
    $self->memory()->add('SUTUnknownError', $SUTUnknownError);
}
#----------------------------------------------------------------------------------------
sub _createSUTSuccessful {
    my $self = shift;
    my ($Shop, $System, $Response) = @_;

    my $aParameterList = [$Shop];
    my $hHeaders = {};
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                 'DE_EPAGES::SageOne::API::Webservice::ShopConfigService', $aParameterList);

    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    $Mockify->addMockWithReturnValueAndParameterCheck('sendGetRequest', $Response,
       [{'string' => 'http://epagesj-endpoint.com'}, {'hashref'=> $hHeaders}]
    );

    $Mockify->addMockWithReturnValueAndParameterCheck('decodeResponse', $Response->content(),
       [{'object'=> 'HTTP::Response'}]
    );

    return $Mockify->getMockObject();
}

sub _createSUTWebserviceError {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopConfigService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    $Mockify->addMock('sendGetRequest', sub{ Error('WebserviceError');});

    return $Mockify->getMockObject();
}

sub _createSUTJsonParserError {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopConfigService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    $Mockify->addMockWithReturnValueAndParameterCheck('sendGetRequest', $self->_createResponse(HTTP_OK, 'abc'), [{'string'=>'http://epagesj-endpoint.com'}, {'hashref' => {}}]);

    $Mockify->addMock('decodeResponse', sub{ Error('JsonParserError');});

    return $Mockify->getMockObject();
}

sub _createSUTUnknownError {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopConfigService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    $Mockify->addMock('sendGetRequest', sub{ Error('Unknown');});

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShopUriBuilder {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop, $System];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::Shop::API::ShopUriBuilder',
                   $aParameterList
              );

    my $hQueryParams = {};
    my $PathTemplate = SAGE_SHOP_CONFIG_PATH_TEMPLATE;
    my $hPathParams = {'shopGuid'=>'MyShopAlias'};
    $Mockify->addMockWithReturnValueAndParameterCheck('buildPrivateUri', 'http://epagesj-endpoint.com', [
        {'string' => $PathTemplate}, {'hashref' => $hPathParams}, {'hashref' => $hQueryParams}
    ]);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createResponse {
    my $self = shift;
    my ($Code, $Content) = @_;
    my $Response = HTTP::Response->new($Code, 'all is fine');
    $Response->header('content-type' => 'application/json');
    $Response->content($Content);
    return $Response;
}
#----------------------------------------------------------------------------------------
sub _createJsonForDTO {
    my $self = shift;
    return {
        'config_id' => '123',
        'shop_id' => 'ConnectedShop',
        'ledger_account_invoice' => '2001'
    }
}

sub _createJsonForNotDTO {
    my $self = shift;
    return {
      'another_shop_id' => 'ConnectedShop'
    };
}
__PACKAGE__->runScript();