#========================================================================================
# package      DE_EPAGES::SageOne::t::unit::ShopConfigService_Put
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::SageOne::API::Webservice::ShopConfigService
#========================================================================================
package DE_EPAGES::SageOne::t::unit::ShopConfigService_Put;
use base DE_EPAGES::Test::API::UnitTest::Test;
use DE_EPAGES::Core::API::Error qw ( Error GetError );
use Test::More;
use Test::Exception;
use HTTP::Status qw (HTTP_OK HTTP_UNPROCESSABLE_ENTITY HTTP_NOT_MODIFIED);
use DE_EPAGES::SageOne::API::Constants qw (SAGE_SHOP_CONFIG_PATH_TEMPLATE SAGE_MAP_FIELDS_TLE);
use HTTP::Response;
use DE_EPAGES::Test::Mock::Mockify qw (WasCalled);
use strict;

sub ErrorDefinition {
    return {
        'JsonParserError' => 'hash- or arrayref expected'
    };
}

sub test {
    my $self = shift;

    $self->_buildMocks();

    $self->test_putShopConfig_ResponseOk();
    $self->test_putShopConfig_ResponseUnprocessableEntity();
    $self->test_putShopConfig_ResponseNotHandled();

    $self->test_putShopConfig_JsonParserError();
    $self->test_putShopConfig_WebserviceError();
    $self->test_putShopConfig_UnknownError();
    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_putShopConfig_ResponseOk {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hContent = {'ledger_account_invoice' => '5555'};
    my $SUT = $self->memory()->get('SUTResponseOk');
    lives_ok( sub { $SUT->updateShopConfig($hContent); }, "$SubTestName - test if updateShopConfig lives");

    my $WasCalled = WasCalled($SUT, 'sendPutRequest');
    is($WasCalled, 1, "$SubTestName - test if sendPutRequest was called");
}

sub test_putShopConfig_ResponseUnprocessableEntity {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hContent = {'ledger_account_invoice' => '5555'};
    my $SUT = $self->memory()->get('SUTResponseUnprocessableEntity');

    throws_ok( sub { $SUT->updateShopConfig($hContent); }, qr/ShopConfigUpdateError: /,
                   "$SubTestName - test if request return status code 422 and contains message error");

    my $Error = GetError();
    my $Expected = $self->_createFieldErrorDTO();

     is_deeply($Error->vars->{'ConfigFieldError'}, [$Expected], "$SubTestName - test if request return error is equal as expected");
}

sub test_putShopConfig_ResponseNotHandled {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hContent = {'ledger_account_invoice' => '5555'};
    my $SUT = $self->memory()->get('SUTResponseNotHandled');

    throws_ok( sub {  $SUT->updateShopConfig($hContent); },
                   qr/WebserviceError: Message=Response status code not handled in updateShopConfig/, ## no critic (ProhibitComplexRegexes)
                   "$SubTestName - test if updateShopConfig throws WebserviceError when the Status Code wasn't handled."
    );
}

sub test_putShopConfig_JsonParserError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hContent = {};
    my $SUT = $self->memory()->get('SUTJsonParserError');

    throws_ok( sub {  $SUT->updateShopConfig($hContent); },
                   qr/WebserviceError: hash- or arrayref expected/,
                   "$SubTestName - test if updateShopConfig throws WebserviceError when JsonParserError occurs"
    );
}

sub test_putShopConfig_WebserviceError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hContent = {};
    my $SUT = $self->memory()->get('SUTWebserviceError');

    throws_ok( sub {  $SUT->updateShopConfig($hContent); },
                   qr/WebserviceError/,
                   "$SubTestName - test if updateShopConfig throws WebserviceError"
    );
}

sub test_putShopConfig_UnknownError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hContent = {};
    my $SUT = $self->memory()->get('SUTUnknownError');

    throws_ok( sub {  $SUT->updateShopConfig($hContent); },
                   qr/Unknown/,
                   "$SubTestName - test if updateShopConfig throws Unknown"
    );
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildMocks {
    my $self = shift;

    my $Shop = $self->mockFactory()->buildShop({
            'GUID' => 'MyShopAlias',
            'RESTAPIURL' => 'http://external/base/url/'
        })->getMockObject();
    my $System = $self->mockFactory->buildSite({
        'EPagesJBaseURL' => 'http://mydomain.epages6.com:8088/rs/'
    })->getMockObject();

    my $SUT_ResponseOk = $self->_createSUT_Successful($Shop, $System, $self->_createResponse(HTTP_OK));
    my $SUT_ResponseSUT_ResponseUnprocessableEntity = $self->_createSUT_ResquestFailed($Shop, $System, $self->_createResponse(HTTP_UNPROCESSABLE_ENTITY, $self->_createJson()));
    my $SUT_ResponseNotHandled = $self->_createSUT_Successful($Shop, $System, $self->_createResponse(HTTP_NOT_MODIFIED));

    my $SUT_JsonParserError = $self->_createSUT_JsonParserError($Shop, $System);
    my $SUT_WebserviceError = $self->_createSUT_WebserviceError($Shop, $System);
    my $SUT_UnknownError = $self->_createSUT_UnknownError($Shop, $System);

    $self->memory()->add('SUTResponseOk', $SUT_ResponseOk);
    $self->memory()->add('SUTResponseUnprocessableEntity', $SUT_ResponseSUT_ResponseUnprocessableEntity);
    $self->memory()->add('SUTResponseNotHandled', $SUT_ResponseNotHandled);

    $self->memory()->add('SUTJsonParserError', $SUT_JsonParserError);
    $self->memory()->add('SUTWebserviceError', $SUT_WebserviceError);
    $self->memory()->add('SUTUnknownError', $SUT_UnknownError);
}
#----------------------------------------------------------------------------------------
sub _createSUT_Successful {
    my $self = shift;
    my ($Shop, $System, $Response) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                 'DE_EPAGES::SageOne::API::Webservice::ShopConfigService', $aParameterList);

    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    my $hHeaders = {};
    my $Url = 'http://epagesj-endpoint.com';
    my $hContent =  {'ledger_account_invoice' => '5555'};
    $Mockify->addMockWithReturnValueAndParameterCheck('sendPutRequest', $Response,
    [
        {'string' => $Url},
        {'hashref' => $hHeaders},
        {'hashref' => $hContent}
    ]);

    $Mockify->addMockWithReturnValueAndParameterCheck('decodeResponse', $Response->content(),
       [{'object'=> 'HTTP::Response'}]
    );

    return $Mockify->getMockObject();
}

sub _createSUT_ResquestFailed {
    my $self = shift;
    my ($Shop, $System, $Response) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                 'DE_EPAGES::SageOne::API::Webservice::ShopConfigService', $aParameterList);

    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    $Mockify->addMock('sendPutRequest', sub{ Error('WebserviceError', {'Status' => HTTP_UNPROCESSABLE_ENTITY, 'Response' => $Response}); });
    $Mockify->addMockWithReturnValueAndParameterCheck('decodeResponse', $Response->content(), [{'object'=> 'HTTP::Response'}]);
    return $Mockify->getMockObject();
}

sub _createSUT_JsonParserError {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopConfigService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    my $Response = $self->_createResponse(HTTP_UNPROCESSABLE_ENTITY, $self->_createJson());
    $Mockify->addMock('sendPutRequest', sub{ Error('WebserviceError', {'Status' => HTTP_UNPROCESSABLE_ENTITY, 'Response' => $Response}); });

    $Mockify->addMock('decodeResponse', sub{ Error('JsonParserError');});

    return $Mockify->getMockObject();
}

sub _createSUT_WebserviceError {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopConfigService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    $Mockify->addMock('sendPutRequest', sub{ Error('WebserviceError');});

    return $Mockify->getMockObject();
}

sub _createSUT_UnknownError {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopConfigService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    $Mockify->addMock('sendPutRequest', sub{ Error('Unknown');});

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShopUriBuilder {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop, $System];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::Shop::API::ShopUriBuilder',
                   $aParameterList
              );

    my $hQueryParams = {};
    my $PathTemplate = SAGE_SHOP_CONFIG_PATH_TEMPLATE;
    my $hPathParams = {'shopGuid'=>'MyShopAlias'};
    $Mockify->addMockWithReturnValueAndParameterCheck('buildPrivateUri', 'http://epagesj-endpoint.com', [
        {'string' => $PathTemplate}, {'hashref' => $hPathParams}, {'hashref' => $hQueryParams}
    ]);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createResponse {
    my $self = shift;
    my ($Code, $Content) = @_;
    my $Response = HTTP::Response->new($Code, 'all is fine');
    $Response->header('content-type' => 'application/json');
    $Response->content($Content);
    return $Response;
}
#----------------------------------------------------------------------------------------
sub _createJson {
    my $self = shift;
    return {
     'fields' => [{
                  'name' => 'ledger_account_invoice',
                  'message' => 'This field doesn\'t exists',
                  'value' => 'Old Value'
                 }]
    }
}

sub _createFieldErrorDTO {
    my $self = shift;
    my $DTO = DE_EPAGES::SageOne::API::DTO::ShopConfigFieldError->new();
    $DTO->formFieldName(SAGE_MAP_FIELDS_TLE->{'ledger_account_invoice'});
    $DTO->name('ledger_account_invoice');
    $DTO->value('Old Value');
    $DTO->message('This field doesn\'t exists');
    return $DTO;
}
__PACKAGE__->runScript();