#========================================================================================
# package      DE_EPAGES::SageOne::t::unit::ShopService_Status
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::SageOne::API::Webservice::ShopService
#========================================================================================
package DE_EPAGES::SageOne::t::unit::ShopService_Status;
use base DE_EPAGES::Test::API::UnitTest::Test;
use DE_EPAGES::SageOne::API::DTO::ShopStatus;
use DE_EPAGES::Core::API::Error qw (Error);
use Test::More;
use Test::Exception;
use HTTP::Response;
use DE_EPAGES::SageOne::API::Constants qw (
    SAGE_SHOP_STATUS_CONNECTED
    SAGE_SYNC_STATUS_IDLE
    SAGE_SYNC_STATUS_NEVER
    SAGE_SHOP_STATUS_DISCONNECTED
);
use strict;

sub test {
    my $self = shift;

    $self->_buildMocks();

    $self->test_getShopStatus_OnBoardedShop();
    $self->test_getShopStatus_NotOnboardedShop();
    $self->test_getShopStatus_UnknowStatusCode();

    $self->test_getShopStatus_WebserviceError();
    $self->test_getShopStatus_ResponseError();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_getShopStatus_OnBoardedShop {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUT_Successful');
    my $DTO = $SUT->getShopStatus();

    is($DTO->shopId(), 'ConnectedShopId', "$SubTestName - test if request was successful and return correct shopId");
    is($DTO->connectionStatus(), SAGE_SHOP_STATUS_CONNECTED, "$SubTestName - test if request was successful and return correct connectionStatus");
    is($DTO->synchronizationStatus(), SAGE_SYNC_STATUS_IDLE, "$SubTestName - test if request was successful and return correct synchronizationStatus");
    is($DTO->lastSuccessfulSynchronization(), '05.02.2016 09:10:54', "$SubTestName - test if request was successful and return correct lastSuccessfulSynchronization");
}

sub test_getShopStatus_NotOnboardedShop {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUT_OnBoardingSuccessful');
    my $DTO = $SUT->getShopStatus();

    is($DTO->shopId(), 'MyShopAlias', "$SubTestName - test if request was successful and return correct shopId");
    is($DTO->connectionStatus(), SAGE_SHOP_STATUS_DISCONNECTED, "$SubTestName - test if request was successful and return correct connectionStatus");
    is($DTO->synchronizationStatus(), SAGE_SYNC_STATUS_IDLE, "$SubTestName - test if request was successful and return correct synchronizationStatus");
    is($DTO->lastSuccessfulSynchronization(), SAGE_SYNC_STATUS_NEVER, "$SubTestName - test if request was successful and return correct lastSuccessfulSynchronization");
}

sub test_getShopStatus_UnknowStatusCode {
     my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUT_ShopStatusError');
    throws_ok(sub { $SUT->getShopStatus(); }, qr/ShopServiceError/, "$SubTestName - test if getShopStatus throws ShopServiceError");
}

sub test_getShopStatus_WebserviceError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUT_WebserviceError');
    throws_ok(sub { $SUT->getShopStatus(); }, qr/WebserviceError/, "$SubTestName - test if getShopStatus throws WebserviceError");
}
sub test_getShopStatus_ResponseError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUT_ResponseError');
    throws_ok(sub { $SUT->getShopStatus(); },
              qr/ResponseError: Missing Field shop_id/,
              "$SubTestName - test if getShopStatus throws ResponseError with missing field shop_id");
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildMocks {
    my $self = shift;

    my $Shop = $self->mockFactory()->buildShop({
            'GUID' => 'MyShopAlias',
            'RESTAPIURL' => 'http://external/base/url/'
        })->getMockObject();
    my $System = $self->mockFactory->buildSite({
        'EPagesJBaseURL' => 'http://unstable-main.epages.com:8088/rs/'
    })->getMockObject();

    my $SUT_Successful = $self->_createSUTSuccessful($Shop, $System, $self->_createResponse200());
    my $SUT_OnBoardingSuccessful = $self->_createSUTSuccessful($Shop, $System, $self->_createResponse204());
    my $SUT_ShopStatusError = $self->_createSUTSuccessful($Shop, $System, $self->_createResponse304());

    my $SUT_WebserviceError = $self->_createSUTWebserviceError($Shop, $System);
    my $SUT_ResponseError = $self->_createSUTResponseError($Shop, $System);

    $self->memory()->add('SUT_Successful', $SUT_Successful);
    $self->memory()->add('SUT_OnBoardingSuccessful', $SUT_OnBoardingSuccessful);
    $self->memory()->add('SUT_ShopStatusError', $SUT_ShopStatusError);

    $self->memory()->add('SUT_WebserviceError', $SUT_WebserviceError);
    $self->memory()->add('SUT_ResponseError', $SUT_ResponseError);

}
#----------------------------------------------------------------------------------------
sub _createSUTSuccessful {
    my $self = shift;
    my ($Shop, $System, $Response) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                 'DE_EPAGES::SageOne::API::Webservice::ShopService', $aParameterList);

    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    $Mockify->addMockWithReturnValueAndParameterCheck('sendGetRequest', $Response,
       [{'string' => 'http://internal-uri.com'}, {'hashref'=> {}}]
    );

    $Mockify->addMockWithReturnValueAndParameterCheck('decodeResponse', $Response->content(),
       [{'object'=> 'HTTP::Response'}]
    );

    return $Mockify->getMockObject();
}

sub _createSUTWebserviceError {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    $Mockify->addMock('sendGetRequest', sub{ Error('WebserviceError');});

    return $Mockify->getMockObject();
}

sub _createSUTResponseError {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    my $Response = $self->_createBadResponse200();
    $Mockify->addMockWithReturnValueAndParameterCheck('sendGetRequest', $Response,
       [{'string' => 'http://internal-uri.com'}, {'hashref'=> {}}]
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('decodeResponse', $Response->content(),
       [{'object'=> 'HTTP::Response'}]
    );

    return $Mockify->getMockObject();
}

sub _createShopUriBuilder {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop, $System];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::Shop::API::ShopUriBuilder',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('buildPrivateUri', 'http://internal-uri.com', [
        {'string' => 'accounting/sageone/{shopGuid}/status'},
        {'hashref' => {'shopGuid'=>'MyShopAlias'}},
        {'hashref' => {}}
    ]);

    return $Mockify->getMockObject();
}

sub _createResponse200 {
    my $self = shift;
    my $Response = HTTP::Response->new(200, 'all is fine'); ## no critic (ProhibitMagicNumbers)
    $Response->header('content-type' => 'application/json');
    my $hJson =  {
         'shop_id' => 'ConnectedShopId',
         'connection_status' => 'connected',
         'synchronization_status' => 'idle',
         'last_successful_synchronization' => '05.02.2016 09:10:54'
       };
    $Response->content($hJson);
    return $Response;
}
sub _createBadResponse200 {
    my $self = shift;
    my $Response = HTTP::Response->new(200, 'all is fine'); ## no critic (ProhibitMagicNumbers)
    $Response->header('content-type' => 'application/json');
    my $hJson =  {
         'another_shod_id' => 'ConnectedShopId',
         'connection_status' => 'connected',
         'synchronization_status' => 'idle',
         'last_successful_synchronization' => '05.02.2016 09:10:54'
       };
    $Response->content($hJson);
    return $Response;
}

sub _createResponse204 {
    my $self = shift;
    my $Response = HTTP::Response->new(204, 'all is fine'); ## no critic (ProhibitMagicNumbers)
    $Response->header('content-type' => 'application/json');
    return $Response;
}

sub _createResponse304 {
    my $self = shift;
    my $Response = HTTP::Response->new(304, 'all is fine'); ## no critic (ProhibitMagicNumbers)
    $Response->header('content-type' => 'application/json');
    return $Response;
}

__PACKAGE__->runScript();