#========================================================================================
# package      DE_EPAGES::SageOne::t::unit::UI_CustomerOrder
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for UI::Shop
#========================================================================================
package DE_EPAGES::SageOne::t::unit::UI_CustomerOrder;
use base DE_EPAGES::Core::API::Test;
use Test::More;
use Test::Exception;
use DE_EPAGES::Test::Mock::Mockify qw (WasCalled);
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::SageOne::API::DTO::AuthUrl;
use DE_EPAGES::SageOne::API::DTO::ShopStatus;

use strict;

sub test {
    my $self = shift;

    $self->test_SageOneSyncOrder();
    $self->test_SageOneSyncOrder_WebServiceError();

    $self->test_SageOneDeleteSalesInvoice();
    $self->test_SageOneDeleteSalesInvoice_WebServiceError();
    $self->test_SageOneDeleteSalesInvoice_BadGateway();

    done_testing();
}

sub test_SageOneSyncOrder {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $OrderService = $self->_createOrderService();
    my $UI_Order = $self->_createUIOrder($OrderService);

    my $Servlet = $self->_createServlet();
    $UI_Order->SageOneSyncOrder($Servlet);

    is( WasCalled($OrderService, 'syncOrder'), 1, "$SubTestName - test if syncOrder was called");
}

sub test_SageOneSyncOrder_WebServiceError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $OrderService = $self->_createOrderService_WebServiceError();
    my $UI_Order = $self->_createUIOrder($OrderService);

    my $Servlet = $self->_createServlet();

    lives_ok( sub { $UI_Order->SageOneSyncOrder($Servlet) }, "$SubTestName - test if syncOrder catches WebServiceError");
    is( WasCalled($OrderService, 'syncOrder'), 1, "$SubTestName - test if syncOrder was called");
}

sub test_SageOneDeleteSalesInvoice {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $OrderService = $self->_createOrderService();
    my $UI_Order = $self->_createUIOrder($OrderService);

    my $Servlet = $self->_createServlet();
    $UI_Order->SageOneDeleteSalesInvoice($Servlet);

    is( WasCalled($OrderService, 'deleteSalesInvoice'), 1, "$SubTestName - test if deleteSalesInvoice was called");
}

sub test_SageOneDeleteSalesInvoice_WebServiceError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $OrderService = $self->_createOrderService_WebServiceError();
    my $UI_Order = $self->_createUIOrder($OrderService);

    my $Servlet = $self->_createServlet();

    lives_ok( sub { $UI_Order->SageOneDeleteSalesInvoice($Servlet) }, "$SubTestName - test if deleteSalesInvoice catches WebServiceError");
    is( WasCalled($OrderService, 'deleteSalesInvoice'), 1, "$SubTestName - test if deleteSalesInvoice was called");
}

sub test_SageOneDeleteSalesInvoice_BadGateway {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $OrderService = $self->_createOrderService_BadGateway();
    my $UI_Order = $self->_createUIOrder($OrderService);

    my $Servlet = $self->_createServlet();

    lives_ok(sub { $UI_Order->SageOneDeleteSalesInvoice($Servlet) }, "$SubTestName - test if deleteSalesInvoice catches WebServiceError");
    is(WasCalled($OrderService, 'deleteSalesInvoice'), 1, "$SubTestName - test if syncOrder was called");
    is($Servlet->vars('SAGEONE_DELETE_SALESINVOICE_FAILED'), 1, "$SubTestName - test if Error was set on Servlet vars")
}

sub _createServlet {
    my $self = shift;
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new( $self->_createCustomerOrder() );
    return $MockifyServlet->getMockObject();
}

sub _createUIOrder {
    my $self = shift;
    my ($OrderService) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::SageOne::UI::CustomerOrder', [] );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createOrderService', $OrderService, [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);
    $Mockify->mock('_getLogger', $self->_createLogger());
    return $Mockify->getMockObject();
}

sub _createOrderService {
    my $self = shift;
    my $aParameterList = [$self->_createShop()];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::OrderService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValue('syncOrder', undef, [{'string'=>'MyOrderGUID'}]);
    $Mockify->addMockWithReturnValue('deleteSalesInvoice', undef, [{'string'=>'MyOrderAlias'}]);
    return $Mockify->getMockObject();
}

sub _createOrderService_WebServiceError {
    my $self = shift;
    my $aParameterList = [$self->_createShop()];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::OrderService',
                   $aParameterList
              );
    $Mockify->addMock('syncOrder', sub{
        Error('WebserviceError');
    });
    $Mockify->addMock('deleteSalesInvoice', sub{
        Error('WebserviceError');
    });
    return $Mockify->getMockObject();
}

sub _createOrderService_BadGateway {
    my $self = shift;
    my $aParameterList = [$self->_createShop()];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::OrderService',
                   $aParameterList
              );
    $Mockify->addMock('syncOrder', sub{
        Error('WebserviceError');
    });
    $Mockify->addMock('deleteSalesInvoice', sub{
        Error('WebserviceError', {'Status' => 502});
    });
    return $Mockify->getMockObject();
}

sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'GUID'=>'MyShopGUID',
        'RESTAPIURL' => 'http://epagesj/base/url/',
    };
    my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    return $MockifyEpagesObject->getMockObject();
}

sub _createCustomerOrder {
    my $self = shift;

    my $hAttributes = {
        'GUID' => 'MyOrderGUID',
        'Shop' => $self->_createShop(),
        'ID' => 123,
        'Site' => $self->_createSite()
    };
    my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes);
    return $MockifyEpagesObject->getMockObject();
}

sub _createSite {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Site', {'ID' => 123, 'GUID'=>'MySiteGUID',});
    return $Mockify->getMockObject();
}

sub _createLogger {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::SageOne::API::Logger');
    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();
