#========================================================================================
# package      DE_EPAGES::SageOne::t::unit::UI_JSON_SageOneOrderStatus
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::SageOne::UI::JSON action SageOneOrderStatus
#========================================================================================
package DE_EPAGES::SageOne::t::unit::UI_JSON_SageOneOrderStatus;
use base DE_EPAGES::Test::API::UnitTest::Test;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Test::Mock::Mockify qw ( GetCallCount GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::More;
use Test::Exception;
use strict;

sub test {
    my $self = shift;

    $self->_buildMocks();

    $self->test_SageOneOrderStatus_happyPath();
    $self->test_SageOneOrderStatus_OrderNotFound_error();
    $self->test_SageOneOrderStatus_UnexpectedError_error();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_SageOneOrderStatus_happyPath {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUT');
    my $Servlet = $self->memory()->get('Servlet');

    $SUT->SageOneOrderStatus($Servlet);

    is(GetCallCount($SUT, 'writeContentOutput'), 1, "$SubTestName - checks if writeContentOutput was called only once");
    my $aReturnParams = GetParametersFromMockifyCall($SUT, 'writeContentOutput');
    my $aExpectedParams = [ {
            'LastStatusChange' => '12.08.1492 13:14', # german datetime format
            'Status' => 'paid',
            'SageInvoiceReference' => 3002,
            'SagePaymentReference' => 1002,
            'StatusMessage' => 'someText',
          } ];

    is_deeply($aReturnParams, $aExpectedParams, "$SubTestName - checks if hash was build as expected");
}
#----------------------------------------------------------------------------------------
sub test_SageOneOrderStatus_OrderNotFound_error {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT_OrderNotFound = $self->memory()->get('SUT_OrderNotFound');
    my $Servlet = $self->memory()->get('Servlet');

    $SUT_OrderNotFound->SageOneOrderStatus($Servlet);

    is(GetCallCount($SUT_OrderNotFound, 'writeContentOutput'), 1, "$SubTestName - checks if writeContentOutput was called only once");
    my $aReturnParams = GetParametersFromMockifyCall($SUT_OrderNotFound, 'writeContentOutput');
    is_deeply($aReturnParams->[0]->{'Status'}, 'not_published', "$SubTestName - checks if hash contains status not published");
}
#----------------------------------------------------------------------------------------
sub test_SageOneOrderStatus_UnexpectedError_error {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUTwithError_UnexpectedError = $self->memory()->get('SUTwithError_UnexpectedError');
    my $Servlet = $self->memory()->get('Servlet');

    throws_ok( sub { $SUTwithError_UnexpectedError->SageOneOrderStatus($Servlet) },
                   qr/UnexpectedError/,
                   "$SubTestName - tests if an error will be rethrown"
         );

}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildMocks {
    my $self = shift;

    my $Shop = $self->_createShop();
    my $Servlet = $self->_createServlet($Shop);

    my $SUT = $self->_createSUT($Servlet);
    my $SUT_OrderNotFound = $self->_createSUT_OrderNotFound($Servlet);
    my $SUTwithError_UnexpectedError = $self->_createSUTwithError($Servlet, 'UnexpectedError');

    $self->memory()->add('Servlet', $Servlet);
    $self->memory()->add('SUT', $SUT);
    $self->memory()->add('SUT_OrderNotFound', $SUT_OrderNotFound);
    $self->memory()->add('SUTwithError_UnexpectedError', $SUTwithError_UnexpectedError);
}
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($Servlet) = @_;

    my $aParameterList = ['Servlet' => $Servlet];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::UI::JSON',
                   $aParameterList
              );
    $Mockify->mock('_createOrderService', $self->_createOrderService(), [{'object' => 'DE_EPAGES::Shop::API::Object::Shop'}]);
    $Mockify->mock('writeContentOutput', 'DummyResponse', ['hashref']);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createSUT_OrderNotFound {
    my $self = shift;
    my ($Servlet) = @_;

    my $aParameterList = ['Servlet' => $Servlet];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::UI::JSON',
                   $aParameterList
              );
    $Mockify->mock('_createOrderService', $self->_createOrderService_OrderNotFound(), [{'object' => 'DE_EPAGES::Shop::API::Object::Shop'}]);
    $Mockify->mock('writeContentOutput', 'DummyResponse', ['hashref']);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createSUTwithError {
    my $self = shift;
    my ($Servlet, $ErrorCode) = @_;

    my $aParameterList = ['Servlet' => $Servlet];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::UI::JSON',
                   $aParameterList
              );
    $Mockify->mock('_createOrderService', sub{ Error($ErrorCode) });
    $Mockify->mock('writeContentOutput', 'DummyResponse', ['hashref']);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createOrderService {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::OrderService',
                   $aParameterList
              );
    my $OrderStatusDto = DE_EPAGES::SageOne::API::DTO::OrderStatus->new();
    $OrderStatusDto->orderId(123456); ## no critic (ProhibitMagicNumbers)
    $OrderStatusDto->sagePaymentReference(1002); ## no critic (ProhibitMagicNumbers)
    $OrderStatusDto->sageSalesInvoiceReference(3002); ## no critic (ProhibitMagicNumbers)
    $OrderStatusDto->shopId('DemoShop');
    $OrderStatusDto->status('paid');
    $OrderStatusDto->statusMessage('someText');
    $OrderStatusDto->statusChanged('1492-08-12 13:14:15'); #mysql timestamp
    $Mockify->mock('getOrderStatus', $OrderStatusDto, [{'int' => 1002}]);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createOrderService_OrderNotFound {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::OrderService',
                   $aParameterList
              );
    my $OrderStatusDto = DE_EPAGES::SageOne::API::DTO::OrderStatus->new();
    $OrderStatusDto->status('not_published');
    $Mockify->mock('getOrderStatus', $OrderStatusDto, [{'int' => 1002}]);

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createServlet {
    my $self = shift;
    my ($Shop) = @_;

    my $hAttributes = {
        'GUID' => '1002',
        'Shop' => $Shop,
        'ID' => 123,
        'Site' => $self->_createSite()
    };

    my $CustomerOrder = $self->mockFactory()->buildCustomerOrder($hAttributes)->getMockObject();
    my $Mockify = DE_EPAGES::Test::Mock::MockifyServlet->new( $CustomerOrder );
    $Mockify->addVar('Locale', 'de_DE');

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;
    my $Mockify =  $self->mockFactory()->buildShop({
            'GUID' => 'MyShopAlias',
            'RESTAPIURL' => 'http://external/base/url/',
            'ID' => 123,
            'Site' => $self->_createSite()
        });
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createSite {
    my $self = shift;

    my $Mockify =  $self->mockFactory()->buildSite({
            'ID' => 123,
            'GUID' => 'MySiteGUID',
        });

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();