#!/bin/bash
#
# Script deletes empty directories in:
#   * Shared/Stores/*/Shops/*/[0-9A-F][0-9A-F][0-9A-F][0-9A-F]/
#   * Shared/WebRoot/*/Shops/*/[0-9A-F][0-9A-F][0-9A-F][0-9A-F]/

# get SCRIPT_NAME
SCRIPT_NAME=$(basename $0)

# set EPAGES_PROJECT to epages6 unless defined
EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}
# source /etc/default/$EPAGES_PROJECT unless $EPAGES is set
[[ -n $EPAGES ]] || . /etc/default/$EPAGES_PROJECT

PID_FILE=/var/run/${SCRIPT_NAME%.sh}.pid
RUN_TIME=7200 #seconds (2 hours)
VERBOSE=false
DRY_RUN=false

########################################################################
# USAGE
########################################################################

usage() {
  [[ -n $@ ]] && echo "Error: $@"
  echo "Usage: $SCRIPT_NAME [ Options ] {-f SHOPFILE | -s 'SHOPGLOB'}"
  echo "Description:"
  echo "  Script deletes empty directories in:"
  echo "    * $EPAGES_STORES/*/Shops/*/[0-9A-F][0-9A-F][0-9A-F][0-9A-F]/"
  echo "    * $EPAGES_WEBROOT/*/Shops/*/[0-9A-F][0-9A-F][0-9A-F][0-9A-F]/"
  echo "Options:"
  echo "  -f SHOPFILE : handle shops from SHOPFILE"
  echo "  -s SHOPGLOB : shop w/ format STORE/Shops/SHOP, Shops/SHOP opt., quoted globs"
  echo "  -t RUN_TIME : max. excecution time of the script, default: $RUN_TIME seconds"
  echo "  -k KEEP_NEW : do not delete empty dirs newer than KEEP_NEW minutes,"
  echo "                default: delete all empty dirs"
  echo "  -p PID_FILE : .pid file for $SCRIPT_NAME, default: $PID_FILE"
  echo "  -n          : dry-run, perform a trial run with no changes made, default: $DRY_RUN"
  echo "  -v          : verbose, show directories to be deleted, default: $VERBOSE"
  echo "  -l          : delete only local files; do nothing if filesystem is not local"
  echo "Examples:"
  echo "  $SCRIPT_NAME -n -s 'StoreXY'"
  echo "  $SCRIPT_NAME -n -s 'StoreXY/Shops/ab*'"
  echo "  $SCRIPT_NAME -n -s 'Store*/Shops/ab*'"
  echo "  $SCRIPT_NAME -n -v -s 'StoreXY/Shops/11111111'"
  exit 1
}

########################################################################
# ARGUMENTS
########################################################################

unset SHOPFILE SHOPGLOB KEEP_NEW ONLY_LOCAL LOCAL_FS_LIST

# get options
while getopts f:s:t:k:p:nvl c ; do
  case $c in
    f) SHOPFILE=$OPTARG ;;
    s) SHOPGLOB=$OPTARG ;;
    t) RUN_TIME=$OPTARG ;;
    k) KEEP_NEW=$OPTARG ;;
    p) PID_FILE=$OPTARG ;;
    n) DRY_RUN=true ;;
    v) VERBOSE=true ;;
    l) ONLY_LOCAL=1 ;;
    *) usage ;;
  esac
done

# check that RUN_TIME/KEEP_NEW consists only of numbers
[[ -n ${RUN_TIME//[0-9]/} ]] && usage "RUN_TIME must be number." 
[[ -n $KEEP_NEW && -n ${KEEP_NEW//[0-9]/} ]] && usage "KEEP_NEW must be number." 

# check SHOPFILE and SHOPGLOB
if [[ -z $SHOPFILE && -z $SHOPGLOB ]] ; then
  usage "Neither -s SHOPGLOB nor -f SHOPFILE is used."
elif [[ -n $SHOPFILE && -n $SHOPGLOB ]] ; then
  usage "Both -s SHOPGLOB and -f SHOPFILE are used."
elif [[ -n $SHOPGLOB ]] ; then
  # either there is one argument
  FIND_STORE=${SHOPGLOB%/*}
  if [[ $FIND_STORE = $SHOPGLOB ]] ; then
    # argument is just STORE
    FIND_STORE=$FIND_STORE/Shops
    # now FIND_STORE is STORE/Shops
    FIND_SHOP='*'
  else
    # argument is STORE/Shops/SHOP
    # then FIND_SHOP is SHOP
    FIND_SHOP=${SHOPGLOB##*/}
  fi
elif [[ -n $SHOPFILE && ! -f $SHOPFILE ]] ; then
  ls $SHOPFILE >/dev/null || usage
fi

if [[ -n $ONLY_LOCAL ]] ; then
  LOCAL_FS_LIST=" "$(ls -d /usr/sbin/mkfs.* /sbin/mkfs.* 2>/dev/null | while read FS
                  do echo -n "${FS##*/mkfs.} " ; done)" "
fi

########################################################################
# FUNCTIONS
########################################################################

drop_shop_entry() {
  local epages_shop=$1
  [[ $DRY_RUN = true ]] && return

  grep -v "^$epages_shop$" $SHOPFILE > $SHOPFILE.~$$~
  cat $SHOPFILE.~$$~ > $SHOPFILE
  rm -f $SHOPFILE.~$$~
}

loop_shopfile() {
  local epages_dir epages_shop

  for epages_shop in $(cat $SHOPFILE) ; do
    # $epages_shop must have non-empty characters
    [[ -n ${epages_shop//[[:space:]]/} ]] || continue
    for epages_dir in $EPAGES_STORES $EPAGES_WEBROOT ; do
      [[ -e $epages_dir/$epages_shop ]] || continue
      if [[ -n $ONLY_LOCAL ]] ; then
        FS=$(find $(readlink -f $epages_dir/$epages_shop) -maxdepth 0 -printf %F)
        if [[ ! $LOCAL_FS_LIST =~ " $FS " ]] ; then
          echo "SKIP: $epages_dir/$epages_shop: filesystem '$FS' not local"
          continue
        fi
      fi
      purge_empty_dirs $epages_dir/$epages_shop
    done
    # delete $epages_shop in $SHOPFILE
    drop_shop_entry $epages_shop
  done
  return 0
}

loop_shopglob() {
  local epages_dir epages_store epages_shop

  for epages_dir in $EPAGES_STORES $EPAGES_WEBROOT ; do
    # find all epages stores with Shops in it (if some)
    for epages_store in $(ls -d $epages_dir/$FIND_STORE) ; do
      if [[ -n $ONLY_LOCAL ]] ; then
        FS=$(find $(readlink -f $epages_store) -maxdepth 0 -printf %F)
        if [[ ! $LOCAL_FS_LIST =~ " $FS " ]] ; then
          echo "SKIP: $epages_store: filesystem '$FS' not local"
          continue
        fi
      fi
      # for each shop in store delete empty GUID directories (if some)
      for epages_shop in $(ls -d $epages_store/$FIND_SHOP) ; do
        purge_empty_dirs $epages_shop
      done
    done
  done
  return 0
}

purge_empty_dirs() {
  local epages_shop=$1

  # ignore precreated shops
  [[ $epages_shop = ${epages_shop##*/PreCreated} ]] || return 0
  # exit if time limit reached
  [[ -e $PID_FILE ]] || exit 0
  # go on if no shop directory found
  pushd $epages_shop >/dev/null 2>&1 || return
  # go on if no GUID directory found
  ls -d [0-9A-F][0-9A-F][0-9A-F][0-9A-F] >/dev/null 2>&1 || { popd >/dev/null; return; }

  # show empty directories
  echo $epages_shop
  if [[ $DRY_RUN = true ]] ; then
    if [[ $VERBOSE = true ]] ; then
      return
    elif [[ -n $KEEP_NEW ]] ; then
      find [0-9A-F][0-9A-F][0-9A-F][0-9A-F] -type d -empty -mmin +$KEEP_NEW
    else
      find [0-9A-F][0-9A-F][0-9A-F][0-9A-F] -type d -empty
    fi
  else
    # delete empty directories in $epages_shop
    [[ $VERBOSE = true ]] && ls -d [0-9A-F][0-9A-F][0-9A-F][0-9A-F]
    if [[ -n $KEEP_NEW ]] ; then
      find [0-9A-F][0-9A-F][0-9A-F][0-9A-F] -type d -mmin +$KEEP_NEW -delete 2>/dev/null
    else
      find [0-9A-F][0-9A-F][0-9A-F][0-9A-F] -type d -delete 2>/dev/null
    fi
  fi
  popd >/dev/null
  return 0
}

########################################################################
# EXECUTION
########################################################################

seconds=$(date +%s)

mkdir -m 755 -p $(dirname $PID_FILE)
touch $PID_FILE || exit 1

# start purge_empty_dirs in the background
if [[ -n $SHOPFILE ]] ; then
  loop_shopfile &
else
  loop_shopglob &
fi

# and wait until purge_empty_dirs has finished (/proc/$! does not exist anymore)
# or purging exceeds $RUN_TIME seconds
WAITED=0
while [[ -d /proc/$! && $WAITED -lt $RUN_TIME ]] ; do
  sleep 2
  WAITED=$(($WAITED + 2))
done

# $PID_FILE is deleted when purge_empty_dirs exceeds $RUN_TIME seconds
[[ $WAITED -ge $RUN_TIME ]] && rm $PID_FILE

# purge_empty_dirs stops (wait is over) when $PID_FILE does not exist anymore
wait

# if $PID_FILE still exists purge_empty_dirs was finished before $RUN_TIME
[[ $DRY_RUN = true ]] && DO=listing || DO=purging
[[ -f $PID_FILE ]] && DONE=finished || DONE=aborted
DURATION=$(($(date +%s) - $seconds))
echo "Exiting: $DO empty directories after $DURATION seconds $DONE."

rm -f $PID_FILE

########################################################################
# END
########################################################################
exit 0
